@@ -1,26 +1,23 @@
 """
 Utilities for parsing a test specification.
-
-
 """
 
 import ast
+import pathlib
+from typing import Any, Dict, Optional, Union
 
 import yaml
 
 
-def load_docstring(filepath):
-    file_contents = ""
+def load_docstring(filepath: Union[str, pathlib.Path]) -> str:
     with open(filepath) as fd:
         file_contents = fd.read()
     module = ast.parse(file_contents)
     docstring = ast.get_docstring(module)
-    if docstring is None:
-        docstring = ""
-    return docstring
+    return docstring or ""
 
 
-def find_yaml_str(s):
+def find_yaml_str(s: str) -> Optional[str]:
     """Loads YAML from docstring."""
     split_lines = s.split("\n")
 
@@ -36,23 +33,23 @@ def find_yaml_str(s):
     return None
 
 
-def load_yaml_from_docstring(docstring):
+def load_yaml_from_docstring(docstring: str) -> Dict[str, Any]:
     found = find_yaml_str(docstring)
     if not found:
-        return None
+        return dict()
     return load_yaml_from_str(found)
 
 
-def load_yaml_from_str(yaml_string):
+def load_yaml_from_str(yaml_string: str) -> Dict[str, Any]:
     try:
-        data = yaml.load(yaml_string, Loader=yaml.SafeLoader)
+        data = dict(yaml.load(yaml_string, Loader=yaml.SafeLoader))
     except yaml.scanner.ScannerError:
         print("WARNING: Unable to parse yaml")
         data = {}
     return data
 
 
-def load_yaml_from_file(filepath):
+def load_yaml_from_file(filepath: Union[str, pathlib.Path]) -> Dict[str, Any]:
     file_contents = ""
     with open(filepath) as fd:
         file_contents = fd.read()
